/* ====================================================================
 * Copyright (c) 2003 The OpenSSL Project.  All rights reserved.
 *
 *
 * This command is intended as a test driver for the FIPS-140 testing
 * lab performing FIPS-140 validation.  It demonstrates the use of the
 * OpenSSL library ito perform a variety of common cryptographic
 * functions.  A power-up self test is demonstrated by deliberately
 * pointing to an invalid executable hash
 *
 * Contributed by Steve Marquess.
 *
 */
#include <stdio.h>
#include <assert.h>
#include <ctype.h>
#include <string.h>
#include <stdlib.h>
#include <signal.h>
#include <openssl/aes.h>
#include <openssl/des.h>
#include <openssl/rsa.h>
#include <openssl/dsa.h>
#include <openssl/hmac.h>
#include <openssl/fips_sha.h>
#include <openssl/md5.h>
#include <openssl/err.h>
#include <openssl/fips.h>
#include <openssl/bn.h>                                                                                          
#include <openssl/rand.h>                                                                                          

int hex2bin(const char *in, unsigned char *out)
    {
    int n1, n2, isodd = 0;
    unsigned char ch;

    n1 = strlen(in);
    if (in[n1 - 1] == '\n')
    n1--;

    if (n1 & 1)
    isodd = 1;

    for (n1=0,n2=0 ; in[n1] && in[n1] != '\n' ; )
    { /* first byte */
    if ((in[n1] >= '0') && (in[n1] <= '9'))
        ch = in[n1++] - '0';
    else if ((in[n1] >= 'A') && (in[n1] <= 'F'))
        ch = in[n1++] - 'A' + 10;
    else if ((in[n1] >= 'a') && (in[n1] <= 'f'))
        ch = in[n1++] - 'a' + 10;
    else
        return -1;
    if(!in[n1])
        {
        out[n2++]=ch;
        break;
        }
    /* If input is odd length first digit is least significant: assumes
     * all digits valid hex and null terminated which is true for the
     * strings we pass.
     */
    if (n1 == 1 && isodd)
        {
        out[n2++] = ch;
        continue;
        }
    out[n2] = ch << 4;
     /* second byte */
    if ((in[n1] >= '0') && (in[n1] <= '9'))
        ch = in[n1++] - '0';
    else if ((in[n1] >= 'A') && (in[n1] <= 'F'))
        ch = in[n1++] - 'A' + 10;
    else if ((in[n1] >= 'a') && (in[n1] <= 'f'))
        ch = in[n1++] - 'a' + 10;
    else
        return -1;
    out[n2++] |= ch;
    }
    return n2;
    }

#ifndef OPENSSL_FIPS
int main(int argc, char *argv[])
    {
    printf("No FIPS support\n");
    return(0);
    }
#else
//#define DEBUG_RSA_SIGNATURE	// Debug RSA 2048 with SHA256 KAT
//#define DETAILED_RSA_SIGNATURE	// Detailed debug for RSA 2048 with SHA256 KAT  
#define SIZE_OF_SIGN 256		// signature size don't comment
/* AES: encrypt and decrypt known plaintext, verify result matches original plaintext
*/
static int FIPS_aes_test(int corrupted)
    {
		if (corrupted) {
			if (FIPS_selftest_aes()!= 0)
				return 1;
			else 
				return 0;
		} else {
			if (FIPS_selftest_aes_cbc() != 0)
				return 1;
			else
				return 0;
		}
    }

/* DES: encrypt and decrypt known plaintext, verify result matches original plaintext
*/
static int FIPS_des_test()
    {
    // if (FIPS_selftest_des()!=0)
    if (FIPS_selftest_des_cbc()!=0)
        {
            return 1;
        }
    else
        {
            return 0;
        }
	}

/* DSA: generate key and sign a known digest, then verify the signature
 * against the digest
*/
static int FIPS_dsa_test()
    {
    DSA *dsa = NULL;
    unsigned char dgst[] = "etaonrishdlc";
    unsigned char sig[256];
    unsigned int siglen;

    ERR_clear_error();
    dsa = DSA_generate_parameters(512,NULL,0,NULL,NULL,NULL,NULL);
    if (!dsa)
	return 0;
    if (!DSA_generate_key(dsa))
	return 0;
    if ( DSA_sign(0,dgst,sizeof(dgst) - 1,sig,&siglen,dsa) != 1 )
	return 0;
    if ( DSA_verify(0,dgst,sizeof(dgst) - 1,sig,siglen,dsa) != 1 )
	return 0;
    DSA_free(dsa);
    return 1;
    }

/* RSA: generate keys and encrypt and decrypt known plaintext, verify result
 * matches the original plaintext
*/
static int FIPS_rsa_test()
    {
    RSA *key;
    unsigned char input_ptext[] = "etaonrishdlc";
    unsigned char ctext[256];
    unsigned char ptext[256];
    int n;

    ERR_clear_error();
    key = RSA_generate_key(1024,65537,NULL,NULL);
    if (!key)
	return 0;
    n = RSA_size(key);
    n = RSA_public_encrypt(sizeof(input_ptext) - 1,input_ptext,ctext,key,RSA_PKCS1_PADDING);
    if (n < 0)
	return 0;
    /* to check encrypt api */
    if (memcmp(ctext, input_ptext, sizeof(input_ptext) - 1) == 0) {
	return (0);
    }

    n = RSA_private_decrypt(n,ctext,ptext,key,RSA_PKCS1_PADDING);
    if (n < 0)
	return 0;
    RSA_free(key);
    if (memcmp(input_ptext,ptext,sizeof(input_ptext) - 1))
        return 0;
    return 1;
    }

/* SHA1: generate hash of known digest value and compare to known
   precomputed correct hash
*/
static int FIPS_sha1_test()
    {
    unsigned char digest[SHA_DIGEST_LENGTH] =
        { 0x11, 0xf1, 0x9a, 0x3a, 0xec, 0x1a, 0x1e, 0x8e, 0x65, 0xd4, 0x9a, 0x38, 0x0c, 0x8b, 0x1e, 0x2c, 0xe8, 0xb3, 0xc5, 0x18 };
    unsigned char str[] = "etaonrishd";

    unsigned char md[SHA_DIGEST_LENGTH];

    ERR_clear_error();
    if (!SHA1(str,sizeof(str) - 1,md)) return 0;
    if (memcmp(md,digest,sizeof(md)))
        return 0;
    return 1;
    }

/* SHA256: generate hash of known digest value and compare to known
   precomputed correct hash
*/
static int FIPS_sha256_test()
    {
    unsigned char digest[SHA256_DIGEST_LENGTH] =
	{0xf5, 0x53, 0xcd, 0xb8, 0xcf, 0x1, 0xee, 0x17, 0x9b, 0x93, 0xc9, 0x68, 0xc0, 0xea, 0x40, 0x91,
	 0x6, 0xec, 0x8e, 0x11, 0x96, 0xc8, 0x5d, 0x1c, 0xaf, 0x64, 0x22, 0xe6, 0x50, 0x4f, 0x47, 0x57};
    unsigned char str[] = "etaonrishd";

    unsigned char md[SHA256_DIGEST_LENGTH];

    ERR_clear_error();
    if (!SHA256(str,sizeof(str) - 1,md)) return 0;
    if (memcmp(md,digest,sizeof(md)))
        return 0;
    return 1;
    }

/* SHA512: generate hash of known digest value and compare to known
   precomputed correct hash
*/
static int FIPS_sha512_test()
    {
    unsigned char digest[SHA512_DIGEST_LENGTH] =
	{0x99, 0xc9, 0xe9, 0x5b, 0x88, 0xd4, 0x78, 0x88, 0xdf, 0x88, 0x5f, 0x94, 0x71, 0x64, 0x28, 0xca,
	 0x16, 0x1f, 0x3d, 0xf4, 0x1f, 0xf3, 0x0f, 0xc5, 0x03, 0x99, 0xb2, 0xd0, 0xe7, 0x0b, 0x94, 0x4a,
	 0x45, 0xd2, 0x6c, 0x4f, 0x20, 0x06, 0xef, 0x71, 0xa9, 0x25, 0x7f, 0x24, 0xb1, 0xd9, 0x40, 0x22,
	 0x49, 0x54, 0x10, 0xc2, 0x22, 0x9d, 0x27, 0xfe, 0xbd, 0xd6, 0xd6, 0xeb, 0x2d, 0x42, 0x1d, 0xa3};
    unsigned char str[] = "etaonrishd";

    unsigned char md[SHA512_DIGEST_LENGTH];

    ERR_clear_error();
    if (!SHA512(str,sizeof(str) - 1,md)) return 0;
    if (memcmp(md,digest,sizeof(md)))
        return 0;
    return 1;
    }

/* HMAC-SHA1: generate hash of known digest value and compare to known
   precomputed correct hash
*/
static int FIPS_hmac_sha1_test()
    {
    unsigned char key[] = "etaonrishd";
    unsigned char iv[] = "Sample text";
    unsigned char kaval[EVP_MAX_MD_SIZE] =
	{0x73, 0xf7, 0xa0, 0x48, 0xf8, 0x94, 0xed, 0xdd, 0x0a, 0xea, 0xea, 0x56, 0x1b, 0x61, 0x2e, 0x70,
	 0xb2, 0xfb, 0xec, 0xc6};

    unsigned char out[EVP_MAX_MD_SIZE];
    unsigned int outlen;

    ERR_clear_error();
    if (!HMAC(EVP_sha1(),key,sizeof(key)-1,iv,sizeof(iv)-1,out,&outlen)) return 0;
    if (memcmp(out,kaval,outlen))
        return 0;
    return 1;
    }

/* HMAC-SHA224: generate hash of known digest value and compare to known
   precomputed correct hash
*/
static int FIPS_hmac_sha224_test()
    {
    unsigned char key[] = "etaonrishd";
    unsigned char iv[] = "Sample text";
    unsigned char kaval[EVP_MAX_MD_SIZE] =
	{0x75, 0x58, 0xd5, 0xbd, 0x55, 0x6d, 0x87, 0x0f, 0x75, 0xff, 0xbe, 0x1c, 0xb2, 0xf0, 0x20, 0x35,
	 0xe5, 0x62, 0x49, 0xb6, 0x94, 0xb9, 0xfc, 0x65, 0x34, 0x33, 0x3a, 0x19};

    unsigned char out[EVP_MAX_MD_SIZE];
    unsigned int outlen;

    ERR_clear_error();
    if (!HMAC(EVP_sha224(),key,sizeof(key)-1,iv,sizeof(iv)-1,out,&outlen)) return 0;
    if (memcmp(out,kaval,outlen))
        return 0;
    return 1;
    }

/* HMAC-SHA256: generate hash of known digest value and compare to known
   precomputed correct hash
*/
static int FIPS_hmac_sha256_test()
    {
    unsigned char key[] = "etaonrishd";
    unsigned char iv[] = "Sample text";
    unsigned char kaval[EVP_MAX_MD_SIZE] =
	{0xe9, 0x17, 0xc1, 0x7b, 0x4c, 0x6b, 0x77, 0xda, 0xd2, 0x30, 0x36, 0x02, 0xf5, 0x72, 0x33, 0x87,
	 0x9f, 0xc6, 0x6e, 0x7b, 0x7e, 0xa8, 0xea, 0xaa, 0x9f, 0xba, 0xee, 0x51, 0xff, 0xda, 0x24, 0xf4};

    unsigned char out[EVP_MAX_MD_SIZE];
    unsigned int outlen;

    ERR_clear_error();
    if (!HMAC(EVP_sha256(),key,sizeof(key)-1,iv,sizeof(iv)-1,out,&outlen)) return 0;
    if (memcmp(out,kaval,outlen))
        return 0;
    return 1;
    }

/* HMAC-SHA384: generate hash of known digest value and compare to known
   precomputed correct hash
*/
static int FIPS_hmac_sha384_test()
    {
    unsigned char key[] = "etaonrishd";
    unsigned char iv[] = "Sample text";
    unsigned char kaval[EVP_MAX_MD_SIZE] =
	{0xb2, 0x9d, 0x40, 0x58, 0x32, 0xc4, 0xe3, 0x31, 0xb6, 0x63, 0x08, 0x26, 0x99, 0xef, 0x3b, 0x10,
	 0xe2, 0xdf, 0xf8, 0xff, 0xc6, 0xe1, 0x03, 0x29, 0x81, 0x2a, 0x1b, 0xac, 0xb0, 0x07, 0x39, 0x08,
	 0xf3, 0x91, 0x35, 0x11, 0x76, 0xd6, 0x4c, 0x20, 0xfb, 0x4d, 0xc3, 0xf3, 0xb8, 0x9b, 0x88, 0x1c};

    unsigned char out[EVP_MAX_MD_SIZE];
    unsigned int outlen;

    ERR_clear_error();
    if (!HMAC(EVP_sha384(),key,sizeof(key)-1,iv,sizeof(iv)-1,out,&outlen)) return 0;
    if (memcmp(out,kaval,outlen))
        return 0;
    return 1;
    }

/* HMAC-SHA512: generate hash of known digest value and compare to known
   precomputed correct hash
*/
static int FIPS_hmac_sha512_test()
    {
    unsigned char key[] = "etaonrishd";
    unsigned char iv[] = "Sample text";
    unsigned char kaval[EVP_MAX_MD_SIZE] =
	{0xcd, 0x3e, 0xb9, 0x51, 0xb8, 0xbc, 0x7f, 0x9a, 0x23, 0xaf, 0xf3, 0x77, 0x59, 0x85, 0xa9, 0xe6,
	 0xf7, 0xd1, 0x51, 0x96, 0x17, 0xe0, 0x92, 0xd8, 0xa6, 0x3b, 0xc1, 0xad, 0x7e, 0x24, 0xca, 0xb1,
	 0xd7, 0x79, 0x0a, 0xa5, 0xea, 0x2c, 0x02, 0x58, 0x0b, 0xa6, 0x52, 0x6b, 0x61, 0x7f, 0xeb, 0x9c,
	 0x47, 0x86, 0x5d, 0x74, 0x2b, 0x88, 0xdf, 0xee, 0x46, 0x69, 0x96, 0x3d, 0xa6, 0xd9, 0x2a, 0x53};

    unsigned char out[EVP_MAX_MD_SIZE];
    unsigned int outlen;

    ERR_clear_error();
    if (!HMAC(EVP_sha512(),key,sizeof(key)-1,iv,sizeof(iv)-1,out,&outlen)) return 0;
    if (memcmp(out,kaval,outlen))
        return 0;
    return 1;
    }

/* MD5: generate hash of known digest value and compare to known
   precomputed correct hash
*/
static int md5_test()
    {
    unsigned char digest[MD5_DIGEST_LENGTH] =
	{ 0x48, 0x50, 0xf0, 0xa3, 0x3a, 0xed, 0xd3, 0xaf, 0x6e, 0x47, 0x7f, 0x83, 0x02, 0xb1, 0x09, 0x68 };
    unsigned char str[] = "etaonrishd";

    unsigned char md[MD5_DIGEST_LENGTH];

    ERR_clear_error();
    if (!MD5(str,sizeof(str) - 1,md))
	return 0;
    if (memcmp(md,digest,sizeof(md)))
	return 0;
    return 1;
    }

/* DH: generate shared parameters
*/
static int dh_test()
    {
    DH *dh;

    ERR_clear_error();
    dh = DH_generate_parameters(256, 2, NULL, NULL);
    if (dh)
        return 1;
    return 0;
    }

/* Zeroize
*/
static int Zeroize()
    {
    RSA *key;
    unsigned char userkey[16] = 
	{ 0x48, 0x50, 0xf0, 0xa3, 0x3a, 0xed, 0xd3, 0xaf, 0x6e, 0x47, 0x7f, 0x83, 0x02, 0xb1, 0x09, 0x68 };
    int i, n;
    
    key = RSA_generate_key(1024,65537,NULL,NULL);
    if (!key)
	return 0;
    n = BN_num_bytes(key->d);

#if 0
    printf(" Generated %d byte RSA private key\n", n);
    printf("\tBN key before overwriting:\n%s\n", BN_bn2hex(key->d));
#endif
    BN_rand(key->d,n*8,-1,0);

#if 0
    printf("\tBN key after overwriting:\n%s\n", BN_bn2hex(key->d));

    printf("\tchar buffer key before overwriting: \n\t\t");
    for(i = 0; i < sizeof(userkey); i++) printf("%02x", userkey[i]);
        printf("\n");
#endif
    RAND_bytes(userkey, sizeof userkey);

#if 0
    printf("\tchar buffer key after overwriting: \n\t\t");
    for(i = 0; i < sizeof(userkey); i++) printf("%02x", userkey[i]);
        printf("\n");
#endif

    return 1;
    }

static int Error;
const char * Fail(const char *msg)
    {
    Error++;
    return msg; 
    }

/*
*RSA signature generation and verification test with SHA256
*/
static int FIPS_signature_SHA256_test(int opt)
    {
    static const unsigned char message[] = "data to sign";
    unsigned char signature[SIZE_OF_SIGN] = { };
    unsigned char BinStoredSignature[SIZE_OF_SIGN] = { } ;
    unsigned char md[SHA256_DIGEST_LENGTH] = { };
    unsigned int slen = 0;
    unsigned int verified = 0;
    RSA *private_key = NULL;
    RSA *public_key = NULL;
    BIO *priv_bio = NULL;
    BIO *pub_bio = NULL;
	int i; 

	unsigned char privkey[] = "-----BEGIN RSA PRIVATE KEY-----\n"
	"MIIEogIBAAKCAQEA6Q/IuOAycuMl4dvmf0xrPlS/SyN7Lae8MYEyzETMHAcxArMK\n"
	"v7vcviIy5bPnRJ5wWs8UwzAa2/Mitcicr13wdCxQ2qTKQUTSekEjUzMo7ClDW7mU\n"
	"lI00wOW1qaYrQvS1t8P6Ycx2OWnnaLek77jvSybWOpV+8DGzOwVabXQloDWCED9N\n"
	"sFna1Q4h37WvQqwDwWLDhjGZl1Szm8JLRXAN70+7NtsKlUV1xLeyb8rVJ52ING3j\n"
	"+jDwAPpzYhE8R98+XOSFrq4+I69X+aksgm03YNzr97yZ9Ca5nJvbKRS0J8E4TD1U\n"
	"OaPVGagAjEP/fr4T/V1q4L0Ht85ZwhHItWsKBQIDAQABAoIBAGt5meiXiuwsD7Ye\n"
	"ACdICFSQJD70N8OQDFsP52sHdtCeCF3kTtqoOV18y9fdpJKiNdHKmRqBvoEbmDAY\n"
	"ORktysrn/Mk3IQYERMOX8kBV4VM2nBdad0gh7efehm2bcWvJCVubBoW/Ee3cBpjl\n"
	"xyu9LqyLVlNhyU3evXZEO7Up0IgWA4FXCjrpZCMJDMMLCT2YreBNBhZa8GYFRUCp\n"
	"+fAdjkOm/wZRiDdK2x9m1BjkuJcPxz/ArR7uc42/hpjCe7ykHlFpZEnw42ik5R+f\n"
	"4Ofvh2auDYJiRYwcdpKF3p7p65S6plxHMveYiM2inhUjtmpAYLP3FIrdH9bgbbGa\n"
	"jZUsWAECgYEA9YaFQ1zxkHJtPRok9T8SIHCKuPEXrbEtBbVjCiwZCoxe7sWwydSk\n"
	"KEdrWT6VHpdj2Fm7bHJ23tr1YEc+34Py9tM91aVvHxDCrpnRV9F5yp4vxignIq4m\n"
	"CxzqsGfAVRodkthrM8jKOYQiI6DSZnNJXXXW6AanBB6JWighjloV/QECgYEA8wEk\n"
	"LmTtir3dQdixm3aqyGvxPygTRivNsXmdjy4/75sSy0dmaC9Qwp6SRPd3iNLeikP4\n"
	"lF4bDURrmbfKaaXr0FwQhcGb2LzTgXklGHmVQPcl8AS/XMHPQhDtFCYb2dacqEG+\n"
	"/Uw+fWSrSee36HLtOa+gmLCQ0MqvXjfc9KVIGQUCgYAl3kWXijJH0R5AGlQw7zVg\n"
	"g3sh4y1/wJ/tDXjlDMAf/F53/U2zE3TuMXXBIgHl4fuoVs6tf0oOPT8zRknQN421\n"
	"vAcBeK6W4ALm4xqCrPBX6+X6bh77HlkHxcH8y0PC3HBLQc3nAsUN8e9CB3/sBUP/\n"
	"gjlPOBdybzNIEIB8cDCMAQKBgDUF3rsw0pQfcZPYkIh3kDRDOVSqjeoVzMAY5URE\n"
	"tza1A5jPb4z0Jg8RQqFBzhdQy7YntqnJDuYKM+BKtiPvTFEehYxp/nOAYDY0FOAO\n"
	"R37xbpm8r3Sx5WNpBhILbrBNn8km72GdrVT3DlkF8p8J1LFtnQE0JaUpVLKeRdwr\n"
	"5FtxAoGAbiodaIxnhEwPhgw6xw+zng572w8WgcvctWcao6YsXPF8ilOLbvyFSAvL\n"
	"jzf/T+BmEIuOV/yTDZlDlum0w9OgiR6jCBIxAySXTKwcKl4LX8cTgEgKiiBWoO9M\n"
	"V9KlF+HPvRPnaYmBa/VKaB+rkWrtp1trx05ilSo4a8/pmLJiOzs=\n"
	"-----END RSA PRIVATE KEY-----\n";

	unsigned char pubkey[] = "-----BEGIN PUBLIC KEY-----\n"
	"MIIBIjANBgkqhkiG9w0BAQEFAAOCAQ8AMIIBCgKCAQEA6Q/IuOAycuMl4dvmf0xr\n"
	"PlS/SyN7Lae8MYEyzETMHAcxArMKv7vcviIy5bPnRJ5wWs8UwzAa2/Mitcicr13w\n"
	"dCxQ2qTKQUTSekEjUzMo7ClDW7mUlI00wOW1qaYrQvS1t8P6Ycx2OWnnaLek77jv\n"
	"SybWOpV+8DGzOwVabXQloDWCED9NsFna1Q4h37WvQqwDwWLDhjGZl1Szm8JLRXAN\n"
	"70+7NtsKlUV1xLeyb8rVJ52ING3j+jDwAPpzYhE8R98+XOSFrq4+I69X+aksgm03\n"
	"YNzr97yZ9Ca5nJvbKRS0J8E4TD1UOaPVGagAjEP/fr4T/V1q4L0Ht85ZwhHItWsK\n"
	"BQIDAQAB\n"
	"-----END PUBLIC KEY-----\n";


	unsigned char  KnownSignature[] = "be2d56a5e42e8c573edcea280505b20c1ad360512b"
	"30fbdaf3015290b30a43c8e9b8bfa23337ccb58000c717946895f71fc4497145fe841b458afb"
	"3b0f3367f8bf2c6ca04be6f9ca35aee07aa9ca2aeb4b2a8a5f43cd565d0f0efc9ae657ee6c10"
	"c83057cdd80c0075ec100480da05bebb372251e9c72b5acb6ff77bd88b23aabd8d6870b54d48"
	"61ab94397f2bcd0f7ea38c6e7282fd23f10414ce0f79fd89a1dabbcc49cbb9e06f6d86a3c678"
	"fd11519fa49188d394b995766cd5be1e7168b827773d1ae7d34796fbf806a24e89a2395b5860"
	"c3da6e6a9a570bf59e502e63c098a9c524f2d4fa63228f0fe4045dd5da7f8609021c6945aeca"
	"dc6852eb25085f"; 

	ERR_clear_error();
    
#ifdef DEBUG_RSA_SIGNATURE
	printf( "\n Populating known RSA Public Key \n");
#endif
	pub_bio = BIO_new(BIO_s_mem());
	if(pub_bio == NULL) {
        ERR_print_errors_fp(stdout);
        return 0;
    }

    BIO_puts(pub_bio, pubkey);

    public_key = (RSA*)PEM_read_bio_RSA_PUBKEY(pub_bio, NULL, NULL, NULL);
    if(public_key == NULL) {
        ERR_print_errors_fp(stdout);
        return(0);
    }

#ifdef DEBUG_RSA_SIGNATURE
	printf( " Populating Known RSA Private Key \n");
#endif
	priv_bio = BIO_new(BIO_s_mem());
   	if(priv_bio == NULL) {
		ERR_print_errors_fp(stdout);
      	return 0;
   	}

	BIO_puts(priv_bio, privkey);

    private_key = (RSA* )PEM_read_bio_RSAPrivateKey(priv_bio, NULL, NULL, NULL);
	if(private_key == NULL) {
      	ERR_print_errors_fp(stdout);
		return(0);
   	}
	
#ifdef DEBUG_RSA_SIGNATURE
	printf( " Creating HASH using SHA256 \n");
#endif

    if (!SHA256(message,strlen(message) - 1 ,md)) return 0;

#ifdef DETAILED_RSA_SIGNATURE 
	printf ("Digest   .............\n"); 
	for (i = 0; i < SHA256_DIGEST_LENGTH; i++)
		printf ("%02x", md[i]); 
	printf ("\n"); 
#endif

#ifdef DEBUG_RSA_SIGNATURE
	printf( " Signing using Known RSA Private Key \n");
#endif
    if(RSA_sign(NID_sha256,  md, strlen(md),
        signature, &slen, private_key) != 1) {
        ERR_print_errors_fp(stdout);
		return(0);
    }

#ifdef DETAILED_RSA_SIGNATURE
	printf(" Generated Signature \n"); 
	for (i = 0; i < slen; i++)
		printf ("%02x", signature[i]); 
	printf("\n"); 
#endif

 	hex2bin(KnownSignature, BinStoredSignature);

#ifdef DETAILED_RSA_SIGNATURE
	printf(" Stored Signature \n"); 
	for (i = 0; i < slen; i++)
		printf ("%02x", BinStoredSignature[i]); 
	printf("\n"); 
#endif

	for (i = 0; i < slen; i++) { 
		if (signature[i]   ==  BinStoredSignature[i]) { 
			continue; 

 	} 		else { 
		   	printf ("Difference at 0th Position = %d\n", i);
			printf (" %02X :: %02X\n", signature[i], BinStoredSignature[i]); 
			return 0;
		} 
	}

	if (opt == 1)
	{
#ifdef DEBUG_RSA_SIGNATURE
	printf( " Corrupting signature \n");
#endif
		signature[ SIZE_OF_SIGN - 10 ] = 'a'; /* corrupting with magic no. 10 */
	}

#ifdef DEBUG_RSA_SIGNATURE
	printf( " Verifying Signature using RSA Public Key \n");
#endif

	verified = RSA_verify(NID_sha256, md,
    strlen(md), signature, slen, public_key);

#ifdef DETAILED_RSA_SIGNATURE
	printf (" signature after verify \n"); 
	for (i = 0; i < slen; i++)
		printf ("%02x", signature[i]); 
	printf("\nMessage digest after verify\n "); 
	for (i = 0; i < SHA256_DIGEST_LENGTH; i++)
		printf ("%02x", md[i]); 
	printf ("\n"); 
#endif

	if ( !private_key) {
    	RSA_free(private_key);
	}
	if ( !public_key) {
	    RSA_free(public_key);
	}

#ifdef DEBUG_RSA_SIGNATURE
    printf( " Verified = %d \n",verified);
#endif

    if(verified)
	    return 1;	/* Return 1 if verification successful */
    else
    	return 0; 	/* Return 0 if verification failed */
    }

int main(int argc,char **argv)
    {
	struct sigaction act;

	memset (&act, '\0', sizeof(act));
	act.sa_handler = SIG_IGN;
	sigaction(SIGINT, &act, NULL);
	sigaction(SIGQUIT, &act, NULL);
	sigaction(SIGTSTP, &act, NULL);

    printf("\tFIPS-mode test application\n\n");

    /* Load entropy from external file, if any */
    RAND_load_file(".rnd", 1024);
    
	if (argv[1]) {
        /* Corrupted KAT tests */
        if (!strcmp(argv[1], "aes")) {
            FIPS_corrupt_aes();
            printf("AES-128,192,256 CBC encryption/decryption with corrupted KAT...");
			printf( FIPS_aes_test(1) ? " successful\n" :  Fail(" FAILED!\n") );
        } else if (!strcmp(argv[1], "des")) {
            FIPS_corrupt_des();
            printf("TDES-CBC encryption/decryption with corrupted KAT...\n");
        } else if (!strcmp(argv[1], "dsa")) {
            FIPS_corrupt_dsa();
            printf("DSA key generation and signature validation with corrupted KAT...\n");
        } else if (!strcmp(argv[1], "rsa")) {
            FIPS_corrupt_rsa();
            printf("RSA key generation and encryption/decryption with corrupted KAT...\n");
        } else if (!strcmp(argv[1], "corrupt_sign")) {
            printf("RSA 2048 with 'SHA256' testing :corrupted KAT...");
            printf( !FIPS_signature_SHA256_test(1) ? " successful\n" :  Fail(" FAILED!\n") );
		} else if (!strcmp(argv[1], "sha1")) {
            FIPS_corrupt_sha1();
            printf("SHA-1 hash with corrupted KAT...\n");
	} else if (!strcmp(argv[1], "rng")) {
	    FIPS_corrupt_rng();
	    printf("RNG test with corrupted KAT...\n");
        } else {
            printf("Bad argument \"%s\"\n", argv[1]);
            exit(1);
        }
        if (!FIPS_mode_set(1))
   	    {
	    ERR_load_crypto_strings();
	    ERR_print_errors(BIO_new_fp(stderr,BIO_NOCLOSE));
            printf("Power-up self test failed\n");
	    exit(1);
	}
        printf("Power-up self test successful\n");
        exit(0);
    }

    /* Non-Approved cryptographic operation
    */
    printf("1. Non-Approved cryptographic operation test...\n");
    printf("\ta. Excluded algorithm (MD5)...");
    printf( md5_test() ? "successful\n" :  Fail("FAILED!\n") );
    printf("\tb. Included algorithm (D-H)...");
    printf( dh_test() ? "successful\n" :  Fail("FAILED!\n") );

    /* Power-up self test
    */
    ERR_clear_error();
    printf("2. Automatic power-up self test...\n");
    printf("\t2.a. FIPS RNG selftest...");
    printf(  FIPS_selftest_x931() ? "successful\n" : Fail("FAILED!\n") );
    printf("\t2.b. FIPS Rand method set...");
    if (!FIPS_mode_set(1))
	{
	ERR_load_crypto_strings();
	ERR_print_errors(BIO_new_fp(stderr,BIO_NOCLOSE));
        printf(Fail("FAILED!\n"));
	exit(1);
	}
    printf("successful\n");

    /* AES encryption/decryption
    */
    printf("3. AES-128,192,256 CBC  encryption/decryption...");
    printf( FIPS_aes_test(0) ? "successful\n" :  Fail("FAILED!\n") );

    /* RSA key generation and encryption/decryption
    */
    printf("4. RSA key generation and encryption/decryption...");
    printf( FIPS_rsa_test() ? "successful\n" :  Fail("FAILED!\n") );

    printf("4.1. RSA 2048 with 'SHA256' testing...");
    printf( FIPS_signature_SHA256_test(0) ? "successful\n" :  Fail("FAILED!\n") );

    /* DES-CBC encryption/decryption
    */
    printf("5. TDES-CBC encryption/decryption...");
    printf( FIPS_des_test() ? "successful\n" :  Fail("FAILED!\n") );

    /* DSA key generation and signature validation
    */

#if 0
    printf("6. DSA key generation and signature validation...");
    printf( FIPS_dsa_test() ? "successful\n" :  Fail("FAILED!\n") );
#endif

    /* SHA-1 hash
    */
    printf("6a. SHA-1 hash...");
    printf( FIPS_sha1_test() ? "successful\n" :  Fail("FAILED!\n") );

    /* SHA-256 hash
    */
    printf("6b. SHA-256 hash...");
    printf( FIPS_sha256_test() ? "successful\n" :  Fail("FAILED!\n") );

    /* SHA-512 hash
    */
    printf("6c. SHA-512 hash...");
    printf( FIPS_sha512_test() ? "successful\n" :  Fail("FAILED!\n") );

    /* HMAC-SHA-1 hash
    */
    printf("6d. HMAC-SHA-1 hash...");
    printf( FIPS_hmac_sha1_test() ? "successful\n" :  Fail("FAILED!\n") );

    /* HMAC-SHA-224 hash
    */
    printf("6e. HMAC-SHA-224 hash...");
    printf( FIPS_hmac_sha224_test() ? "successful\n" :  Fail("FAILED!\n") );

    /* HMAC-SHA-256 hash
    */
    printf("6f. HMAC-SHA-256 hash...");
    printf( FIPS_hmac_sha256_test() ? "successful\n" :  Fail("FAILED!\n") );

    /* HMAC-SHA-384 hash
    */
    printf("6g. HMAC-SHA-384 hash...");
    printf( FIPS_hmac_sha384_test() ? "successful\n" :  Fail("FAILED!\n") );

    /* HMAC-SHA-512 hash
    */
    printf("6h. HMAC-SHA-512 hash...");
    printf( FIPS_hmac_sha512_test() ? "successful\n" :  Fail("FAILED!\n") );

    /* Non-Approved cryptographic operation
    */
    printf("7. Non-Approved cryptographic operation test...\n");
    printf("\ta. Excluded algorithm (MD5)...");

#ifndef FABOS
    printf( md5_test() ? Fail("passed INCORRECTLY!\n")
	    : "failed as expected\n" );
#else
    printf("Not executed\n");
#endif

    printf("\tb. Included algorithm (D-H)...");
    printf( dh_test() ? "successful as expected\n"
	    : Fail("failed INCORRECTLY!\n") );

    /* Zeroization
    */
    printf("8. Zero-ization...");
	printf(Zeroize() ? "Successful\n" : Fail("FAILED!\n") );

    printf("\nAll tests completed with %d errors\n", Error);

#ifdef FABOS
    return (Error);
#else
    return 0;
#endif
    }
#endif
