/* ====================================================================
 * Copyright (c) 2011 The OpenSSL Project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer. 
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. All advertising materials mentioning features or use of this
 *    software must display the following acknowledgment:
 *    "This product includes software developed by the OpenSSL Project
 *    for use in the OpenSSL Toolkit. (http://www.openssl.org/)"
 *
 * 4. The names "OpenSSL Toolkit" and "OpenSSL Project" must not be used to
 *    endorse or promote products derived from this software without
 *    prior written permission. For written permission, please contact
 *    openssl-core@openssl.org.
 *
 * 5. Products derived from this software may not be called "OpenSSL"
 *    nor may "OpenSSL" appear in their names without prior written
 *    permission of the OpenSSL Project.
 *
 * 6. Redistributions of any form whatsoever must retain the following
 *    acknowledgment:
 *    "This product includes software developed by the OpenSSL Project
 *    for use in the OpenSSL Toolkit (http://www.openssl.org/)"
 *
 * THIS SOFTWARE IS PROVIDED BY THE OpenSSL PROJECT ``AS IS'' AND ANY
 * EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE OpenSSL PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */

#define OPENSSL_FIPSAPI

#include <openssl/crypto.h>
#include <openssl/rand.h>
#include <openssl/err.h>
#include <openssl/fips.h>
#include <openssl/fips_rand.h>
#include "e_os.h"

/* FIPS API for PRNG use. Similar to RAND functionality but without
 * ENGINE and additional checking for non-FIPS rand methods.
 */

static const RAND_METHOD *fips_rand_meth = NULL;
static int fips_approved_rand_meth = 0;
static int fips_rand_bits = 0;

/* Allows application to override number of bits and uses non-FIPS methods */
void FIPS_rand_set_bits(int nbits)
	{
	fips_rand_bits = nbits;
	}

int FIPS_rand_set_method(const RAND_METHOD *meth)
	{
	if (!fips_rand_bits)
		{
		if (meth == FIPS_drbg_method())
			fips_approved_rand_meth = 1;
		else if (meth == FIPS_x931_method())
			fips_approved_rand_meth = 2;
		else
			{
			fips_approved_rand_meth = 0;
			if (FIPS_module_mode())
				{
				FIPSerr(FIPS_F_FIPS_RAND_SET_METHOD,
						FIPS_R_NON_FIPS_METHOD);
				return 0;
				}
			}
		}
	fips_rand_meth = meth;
	return 1;
	}

const RAND_METHOD *FIPS_rand_get_method(void)
	{
	return fips_rand_meth;
	}

void FIPS_rand_seed(const void *buf, int num)
	{
	if (!fips_approved_rand_meth && FIPS_module_mode())
		{
		FIPSerr(FIPS_F_FIPS_RAND_SEED, FIPS_R_NON_FIPS_METHOD);
		return;
		}
	if (fips_rand_meth && fips_rand_meth->seed)
		fips_rand_meth->seed(buf,num);
	}

void FIPS_rand_add(const void *buf, int num, double entropy)
	{
	if (!fips_approved_rand_meth && FIPS_module_mode())
		{
		FIPSerr(FIPS_F_FIPS_RAND_ADD, FIPS_R_NON_FIPS_METHOD);
		return;
		}
	if (fips_rand_meth && fips_rand_meth->add)
		fips_rand_meth->add(buf,num,entropy);
	}

int FIPS_rand_bytes(unsigned char *buf, int num)
	{
	if (!fips_approved_rand_meth && FIPS_module_mode())
		{
		FIPSerr(FIPS_F_FIPS_RAND_BYTES, FIPS_R_NON_FIPS_METHOD);
		return 0;
		}
	if (fips_rand_meth && fips_rand_meth->bytes)
		return fips_rand_meth->bytes(buf,num);
	return 0;
	}

int FIPS_rand_pseudo_bytes(unsigned char *buf, int num)
	{
	if (!fips_approved_rand_meth && FIPS_module_mode())
		{
		FIPSerr(FIPS_F_FIPS_RAND_PSEUDO_BYTES, FIPS_R_NON_FIPS_METHOD);
		return 0;
		}
	if (fips_rand_meth && fips_rand_meth->pseudorand)
		return fips_rand_meth->pseudorand(buf,num);
	return -1;
	}

int FIPS_rand_status(void)
	{
	if (!fips_approved_rand_meth && FIPS_module_mode())
		{
		FIPSerr(FIPS_F_FIPS_RAND_STATUS, FIPS_R_NON_FIPS_METHOD);
		return 0;
		}
	if (fips_rand_meth && fips_rand_meth->status)
		return fips_rand_meth->status();
	return 0;
	}

/* Return instantiated strength of PRNG. For DRBG this is an internal
 * parameter. For X9.31 PRNG it is 80 bits (from SP800-131). Any other
 * type of PRNG is not approved and returns 0 in FIPS mode and maximum
 * 256 outside FIPS mode.
 */

int FIPS_rand_strength(void)
	{
	if (fips_rand_bits)
		return fips_rand_bits;
	if (fips_approved_rand_meth == 1)
		return FIPS_drbg_get_strength(FIPS_get_default_drbg());
	else if (fips_approved_rand_meth == 2)
		return 80;
	else if (fips_approved_rand_meth == 0)
		{
		if (FIPS_module_mode())
			return 0;
		else
			return 256;
		}
	return 0;
	}

#ifdef IKE_DRBG_NO_FIPS

/*
 * Below code has been added to avoid compiling OpenSSL code 
 * with OPENSSL_FIPS environment variable as OpenSSL capable FIPS 
 * is not supported for the DP processor i.e. we cannot set FIPs mode
 * in DP.
 */ 

int FIPS_module_mode(void)
   {
   return 1;
   }

/* 
 * Below code has been taken from the file "rand_lib.c" from the OpenSSL code.
 * We will not use "RAND_init_fips" and instead replicate its functionality 
 * in our source to allow us to provide our own entropy, 
 * Related code changes done:
 * 1> Error logs in the function "RAND_init_fips" have been changed to reflect
 * this module and function. 
 * 2> Removed static access.
 */

#ifdef OPENSSL_FIPS
   
/* FIPS DRBG initialisation code. This sets up the DRBG for use by the
 * rest of OpenSSL. 
 */

/* Entropy gatherer: use standard OpenSSL PRNG to seed (this will gather
 * entropy internally through RAND_poll().
 */

static size_t drbg_get_entropy(DRBG_CTX *ctx, unsigned char **pout,
                                int entropy, size_t min_len, size_t max_len)
        {
	/* Round up request to multiple of block size */
	min_len = ((min_len + 19) / 20) * 20;
	*pout = OPENSSL_malloc(min_len);
	if (!*pout)
		return 0;
	if (RAND_SSLeay()->bytes(*pout, min_len) <= 0)
		{
		OPENSSL_free(*pout);
		*pout = NULL;
		return 0;
		}
        return min_len;
        }

static void drbg_free_entropy(DRBG_CTX *ctx, unsigned char *out, size_t olen)
	{
	if (out)
		{
		OPENSSL_cleanse(out, olen);
		OPENSSL_free(out);
		}
	}

/* Set "additional input" when generating random data. This uses the
 * current PID, a time value and a counter.
 */

size_t drbg_get_adin(DRBG_CTX *ctx, unsigned char **pout)
    	{
	/* Use of static variables is OK as this happens under a lock */
	static unsigned char buf[16];
	static unsigned long counter;
	FIPS_get_timevec(buf, &counter);
	*pout = buf;
	return sizeof(buf);
	}

/* RAND_add() and RAND_seed() pass through to OpenSSL PRNG so it is 
 * correctly seeded by RAND_poll().
 */

int drbg_rand_add(DRBG_CTX *ctx, const void *in, int inlen,
				double entropy)
	{
	RAND_SSLeay()->add(in, inlen, entropy);
	return 1;
	}

int drbg_rand_seed(DRBG_CTX *ctx, const void *in, int inlen)
	{
	RAND_SSLeay()->seed(in, inlen);
	return 1;
	}

#ifndef OPENSSL_DRBG_DEFAULT_TYPE
#define OPENSSL_DRBG_DEFAULT_TYPE	NID_aes_256_ctr
#endif
#ifndef OPENSSL_DRBG_DEFAULT_FLAGS
#define OPENSSL_DRBG_DEFAULT_FLAGS	DRBG_FLAG_CTR_USE_DF
#endif 

int fips_drbg_type = OPENSSL_DRBG_DEFAULT_TYPE;
int fips_drbg_flags = OPENSSL_DRBG_DEFAULT_FLAGS;

void RAND_set_fips_drbg_type(int type, int flags)
	{
	fips_drbg_type = type;
	fips_drbg_flags = flags;
	}

int RAND_init_fips(void)
	{
	DRBG_CTX *dctx;
	size_t plen;
	unsigned char pers[32], *p;
	dctx = FIPS_get_default_drbg();
        if (FIPS_drbg_init(dctx, fips_drbg_type, fips_drbg_flags) <= 0)
		{
		FIPSerr(FIPS_F_FIPS_RAND_SET_METHOD, FIPS_R_ERROR_INITIALISING_DRBG);
		return 0;
		}
		
        FIPS_drbg_set_callbacks(dctx,
				drbg_get_entropy, drbg_free_entropy, 20,
				drbg_get_entropy, drbg_free_entropy);
	FIPS_drbg_set_rand_callbacks(dctx, drbg_get_adin, 0,
					drbg_rand_seed, drbg_rand_add);
	/* Personalisation string: a string followed by date time vector */
	strcpy((char *)pers, "OpenSSL DRBG2.0");
	plen = drbg_get_adin(dctx, &p);
	memcpy(pers + 16, p, plen);

        if (FIPS_drbg_instantiate(dctx, pers, sizeof(pers)) <= 0)
		{
		FIPSerr(FIPS_F_FIPS_RAND_SET_METHOD, FIPS_R_ERROR_INSTANTIATING_DRBG);
		return 0;
		}
        FIPS_rand_set_method(FIPS_drbg_method());
	return 1;
	}

#endif	/*	OPENSSL_FIPS	*/
#endif	/*	IKE_DRBG_NO_FIPS	*/
